cache_dir = "~\\AppData\\Local\\Nexus\\Cache"
virtualenvs_create = True
virtualenvs_clear = False
virtualenvs_upgradepip = False
virtualenvs_in_project = True
virtualenvs_symlinks = False
virtualenvs_system_site_packages = False
virtualenvs_foldername = ".venv"
proxy = ""
threading = True
threadmax = 4
#virtualenvs.prompt = "{project_name}"
logs_display = True
logs_full = True
logs_date = True
logs_truecolor = False
project_changelog = True
project_deep_search = 3
publish_url = "https://pypi.org/legacy/"
metadata_source = "https://pypi.org"
repo_create = True
repo_commit = "{version} - {date}"
repo_branch_autocreate = True